"""bluepyefe.extractor tests"""

import unittest
import glob
import json

import bluepyefe.extract
import bluepyefe.tools
from tests.utils import download_sahp_datafiles


def get_config(absolute_amplitude=False):

    interesting_efeatures = {
        "Spikecount": {},
        "mean_frequency": {},
        "ISI_CV": {},
        "AP1_amp": {},
        "AP_width": {},
    }

    files_metadata1 = []
    for file in glob.glob("./tests/exp_data/B6/B6_Ch0_IDRest_*.ibw"):
        files_metadata1.append(
            {
                "i_file": file,
                "v_file": file.replace("Ch0", "Ch3"),
                "i_unit": "pA",
                "v_unit": "mV",
                "t_unit": "s",
                "dt": 0.00025,
                "ljp": 14.0,
            }
        )

    # Do the same for the second cell
    files_metadata2 = []
    for file in glob.glob("./tests/exp_data/B8/B8_Ch0_IDRest_*.ibw"):
        files_metadata2.append(
            {
                "i_file": file,
                "v_file": file.replace("Ch0", "Ch3"),
                "i_unit": "pA",
                "v_unit": "mV",
                "t_unit": "s",
                "dt": 0.00025,
                "ljp": 14.0,
            }
        )

    files_metadata = {
        "MouseNeuron1": {"IDRest": files_metadata1},
        "MouseNeuron2": {"IDRest": files_metadata2},
    }

    if absolute_amplitude:
        targets = {
            "IDRest": {
                "amplitudes": [0.15, 0.25],
                "tolerances": [0.05],
                "efeatures": interesting_efeatures,
                "location": "soma",
            }
        }

    else:
        targets = {
            "IDRest": {
                "amplitudes": [150, 200, 250],
                "tolerances": [20.0],
                "efeatures": interesting_efeatures,
                "location": "soma",
            }
        }

    return files_metadata, bluepyefe.extract.convert_legacy_targets(targets)


class ExtractorTest(unittest.TestCase):
    def test_extract(self):

        files_metadata, targets = get_config()

        cells = bluepyefe.extract.read_recordings(files_metadata=files_metadata)

        cells = bluepyefe.extract.extract_efeatures_at_targets(
            cells=cells, targets=targets
        )

        bluepyefe.extract.compute_rheobase(cells, protocols_rheobase=["IDRest"])

        self.assertEqual(len(cells), 2)
        self.assertEqual(len(cells[0].recordings), 5)
        self.assertEqual(len(cells[1].recordings), 5)

        self.assertLess(abs(cells[0].rheobase - 0.119), 0.01)
        self.assertLess(abs(cells[1].rheobase - 0.0923), 0.01)

        protocols = bluepyefe.extract.group_efeatures(
            cells,
            targets,
            use_global_rheobase=True,
            protocol_mode="mean"
        )

        _ = bluepyefe.extract.create_feature_protocol_files(
            cells=cells, protocols=protocols, output_directory="MouseCells"
        )

        for protocol in protocols:
            if protocol.name == "IDRest" and protocol.amplitude == 250.:
                for target in protocol.feature_targets:
                    if target.efel_feature_name == "Spikecount":
                        self.assertEqual(target.mean, 78.5)
                        self.assertEqual(target.std, 3.5)
                        break

        bluepyefe.extract.plot_all_recordings_efeatures(
            cells, protocols, output_dir="MouseCells/"
        )

        with open("MouseCells/features.json") as fp:
            features = json.load(fp)
        with open("MouseCells/protocols.json") as fp:
            protocols = json.load(fp)

        self.assertEqual(len(features), len(protocols))

    def test_extract_auto_fail_rheobase(self):

        files_metadata, _ = get_config()

        efeatures, protocol_definitions, current = bluepyefe.extract.extract_efeatures(
            output_directory="./",
            files_metadata=files_metadata,
            rheobase_strategy="flush",
            rheobase_settings={"upper_bound_spikecount": 4}
        )

        self.assertEqual(len(efeatures), 0)

    def test_extract_auto(self):

        files_metadata, _ = get_config()

        auto_targets = bluepyefe.auto_targets.default_auto_targets()

        cells = bluepyefe.extract.read_recordings(
            files_metadata,
            recording_reader=None,
            map_function=map,
            efel_settings=bluepyefe.tools.DEFAULT_EFEL_SETTINGS
        )

        for cell in cells:
            cell.rheobase = 0.07
            cell.compute_relative_amp()

        recordings = []
        for c in cells:
            recordings += c.recordings

        for i in range(len(auto_targets)):
            auto_targets[i].select_ecode_and_amplitude(recordings)

        # Extract the efeatures and group them around the preset of targets.
        targets = []
        for at in auto_targets:
            targets += at.generate_targets()

        self.assertEqual(len(targets), 48)

    def test_extract_absolute(self):

        files_metadata, targets = get_config(True)

        cells = bluepyefe.extract.read_recordings(files_metadata=files_metadata)

        cells = bluepyefe.extract.extract_efeatures_at_targets(
            cells=cells, targets=targets
        )

        self.assertEqual(len(cells), 2)
        self.assertEqual(len(cells[0].recordings), 5)
        self.assertEqual(len(cells[1].recordings), 5)

        self.assertEqual(cells[0].rheobase, None)
        self.assertEqual(cells[1].rheobase, None)

        protocols = bluepyefe.extract.group_efeatures(
            cells,
            targets,
            absolute_amplitude=True,
            protocol_mode="mean"
        )

        _ = bluepyefe.extract.create_feature_protocol_files(
            cells=cells, protocols=protocols, output_directory="MouseCells"
        )

        for cell in cells:
            for r in cell.recordings:
                print(r.amp, r.efeatures)

        for protocol in protocols:
            if protocol.name == "IDRest" and protocol.amplitude == 0.25:
                for target in protocol.feature_targets:
                    if target.efel_feature_name == "Spikecount":
                        self.assertEqual(target.mean, 76.5)
                        self.assertAlmostEqual(target.std, 5.590169, 4)
                        break

        bluepyefe.extract.plot_all_recordings_efeatures(
            cells, protocols, output_dir="MouseCells/"
        )

        with open("MouseCells/features.json") as fp:
            features = json.load(fp)
        with open("MouseCells/protocols.json") as fp:
            protocols = json.load(fp)

        self.assertEqual(len(features), len(protocols))




if __name__ == "__main__":
    unittest.main()



"""bluepyefe.cell tests"""

import unittest
import pytest

import bluepyefe.cell
import bluepyefe.recording


class EfelSettingTest(unittest.TestCase):
    def setUp(self):

        self.cell = bluepyefe.cell.Cell(name="MouseNeuron")

        file_metadata = {
            "i_file": "./tests/exp_data/B95_Ch0_IDRest_107.ibw",
            "v_file": "./tests/exp_data/B95_Ch3_IDRest_107.ibw",
            "i_unit": "pA",
            "v_unit": "mV",
            "t_unit": "s",
            "dt": 0.00025,
            "ljp": 14.0,
        }

        self.cell.read_recordings(
            protocol_data=[file_metadata],
            protocol_name="IDRest"
        )

    def test_efel_threshold(self):

        self.cell.recordings[0].efeatures = {}

        self.cell.extract_efeatures(
            protocol_name="IDRest",
            efeatures=["Spikecount", "AP1_amp"],
            efel_settings={'Threshold': 40.}
        )

        recording = self.cell.recordings[0]
        self.assertEqual(recording.efeatures["Spikecount"], 0.)
        self.assertLess(abs(recording.efeatures["AP1_amp"] - 66.68), 0.01)

    def test_efel_strictstim(self):

        self.cell.recordings[0].efeatures = {}

        self.cell.extract_efeatures(
            protocol_name="IDRest",
            efeatures=["Spikecount"],
            efel_settings={
                'stim_start': 0,
                'stim_end': 100,
                'strict_stiminterval': True
            }
        )

        self.assertEqual(self.cell.recordings[0].efeatures["Spikecount"], 0.)

    def test_efel_threshold(self):

        self.cell.recordings[0].efeatures = {}

        self.cell.extract_efeatures(
            protocol_name="IDRest",
            efeatures=["Spikecount"],
            efel_settings={'Threshold': 40}
        )

        recording = self.cell.recordings[0]
        self.assertEqual(recording.efeatures["Spikecount"], 0.)

    def test_efel_incorrect_threshold(self):

        self.cell.recordings[0].efeatures = {}

        with pytest.raises(ValueError):
            self.cell.extract_efeatures(
                protocol_name="IDRest",
                efeatures=["Spikecount"],
                efel_settings={'Threshold': ["40."]}
            )

if __name__ == "__main__":
    unittest.main()



"""bluepyefe.nwbreader tests"""
import unittest
import h5py
from pathlib import Path
from bluepyefe.reader import csv_lccr_reader


class TestCSVLCCRReaders(unittest.TestCase):
    def setUp(self):
        self.test_data = {
            'filepath': './tests/exp_data/csv_lccr/dummy/dummy_ch1_cols.txt',
            'protocol_name': 'Step',
            "dt": 0.1,
            "amplitudes": [10, -10, 20, -20, 30, -30, 40, -40, 50, -50, 60, -60, 70, -70, 80, -80, 90, -90, 100, -100, 150, 200, 250, 300, 400, 500, 600],
            "ljp": 14.0,
            "v_file": "dummy",
            "i_unit": "pA",
            "v_unit": "mV",
            "t_unit": "ms",
            "ton": 200,
            "toff": 800,
            "hypamp": -20,
            "remove_last_100ms": True,
        }

    def test_csv_lccr_reader(self):
        filepath = Path(self.test_data['filepath'])
        self.assertTrue(filepath.is_file(), f"{filepath} is not a valid file")

        result = csv_lccr_reader(self.test_data)
        self.assertIsInstance(result, list, f"Result for {filepath} should be a list")
        self.assertEqual(len(result), 27, f"Result for {filepath} should have 27 entries")

        for entry in result:
            self.assertIn('filename', entry)
            self.assertIn('voltage', entry)
            self.assertIn('current', entry)
            self.assertIn('t', entry)
            self.assertIn('dt', entry)
            self.assertIn('ton', entry)
            self.assertIn('toff', entry)
            self.assertIn('amp', entry)
            self.assertIn('hypamp', entry)
            self.assertIn('ljp', entry)
            self.assertIn('i_unit', entry)
            self.assertIn('v_unit', entry)
            self.assertIn('t_unit', entry)

    def test_csv_lccr_reader_empty_amplitudes(self):
        test_data = self.test_data.copy()
        test_data['amplitudes'] = []
        result = csv_lccr_reader(test_data)
        self.assertEqual(len(result), 0, "Result should be an empty list when amplitudes are empty")

    def test_csv_lccr_reader_file_not_found(self):
        test_data = self.test_data.copy()
        test_data['filepath'] = './non_existent_file.txt'
        with self.assertRaises(FileNotFoundError):
            csv_lccr_reader(test_data)

    def test_csv_lccr_reader_remove_last_100ms(self):
        test_data = self.test_data.copy()
        test_data['remove_last_100ms'] = True

        result = csv_lccr_reader(test_data)

        original_length = 14000
        expected_length = original_length - int(100 / test_data['dt'])

        for entry in result:
            self.assertEqual(len(entry['t']), expected_length, "Time array length should be reduced by 100 ms")
            self.assertEqual(len(entry['voltage']), expected_length, "Voltage array length should be reduced by 100 ms")
            self.assertEqual(len(entry['current']), expected_length, "Current array length should be reduced by 100 ms")


if __name__ == '__main__':
    unittest.main()


"""Utils"""

import urllib.request
import shutil
from pathlib import Path

def download_datafiles(pathname, channels, numbers, output_dir, gb_url):
    paths = [f"{pathname}_{ch}_{n}.ibw" for ch in channels for n in numbers]

    Path(output_dir).mkdir(exist_ok=True, parents=True)
    for path in paths:
        output_path = f"{output_dir}{path}"
        if not Path(output_path).is_file():
            with urllib.request.urlopen(f"{gb_url}{path}") as response, open(output_path, "wb") as out_file:
                shutil.copyfileobj(response, out_file)

def download_sahp_datafiles():
    """Download data files for sAHP and IDthresh traces."""
    output_dir = "./tests/exp_data/X/"
    gb_url = "https://raw.githubusercontent.com/BlueBrain/SSCxEModelExamples/main/feature_extraction/input-traces/C060109A1-SR-C1/"
    sahp_pathname = "X_sAHP"
    sahp_ch = ["ch0", "ch1"]
    sahp_numbers = list(range(320, 326))
    idthresh_pathname = "X_IDthresh"
    idthresh_ch = ["ch0", "ch1"]
    idthresh_numbers = list(range(349, 358)) + list(range(362, 371))

    download_datafiles(sahp_pathname, sahp_ch, sahp_numbers, output_dir, gb_url)
    download_datafiles(idthresh_pathname, idthresh_ch, idthresh_numbers, output_dir, gb_url)

def download_apthresh_datafiles():
    """Download data files for APThreshold and IDthresh traces."""
    output_dir = "./tests/exp_data/X/"
    gb_url = "https://raw.githubusercontent.com/BlueBrain/SSCxEModelExamples/main/feature_extraction/input-traces/C060109A1-SR-C1/"
    apthresh_pathname = "X_APThreshold"
    apthresh_ch = ["ch0", "ch1"]
    apthresh_numbers = list(range(254, 257))
    idthresh_pathname = "X_IDthresh"
    idthresh_ch = ["ch0", "ch1"]
    idthresh_numbers = list(range(349, 358)) + list(range(362, 371))

    download_datafiles(apthresh_pathname, apthresh_ch, apthresh_numbers, output_dir, gb_url)
    download_datafiles(idthresh_pathname, idthresh_ch, idthresh_numbers, output_dir, gb_url)



"""bluepyefe.nwbreader tests"""
import unittest
import h5py
from pathlib import Path
from bluepyefe.reader import nwb_reader


class TestNWBReaders(unittest.TestCase):
    def setUp(self):
        self.test_data = {
            'filepath': './tests/exp_data/hippocampus-portal/99111002.nwb',
            'protocol_name': 'Step',
        }

    def test_nwb_reader(self):
        filepath = Path(self.test_data['filepath'])
        self.assertTrue(filepath.is_file(), f"{filepath} is not a valid file")

        result = nwb_reader(self.test_data)

        self.assertIsInstance(result, list, f"Result for {filepath} should be a list")
        self.assertEqual(len(result), 16, f"Result for {filepath} should have 16 entries")

        for entry in result:
            self.assertIn('voltage', entry)
            self.assertIn('current', entry)
            self.assertIn('dt', entry)
            self.assertIn('id', entry)
            self.assertIn('i_unit', entry)
            self.assertIn('v_unit', entry)
            self.assertIn('t_unit', entry)


if __name__ == '__main__':
    unittest.main()


"""bluepyefe.cell tests"""

import unittest

from numpy.testing import assert_array_almost_equal
from pytest import approx

import bluepyefe.cell
import bluepyefe.recording
from bluepyefe.reader import igor_reader
from bluepyefe.ecode.step import Step


class RecordingTest(unittest.TestCase):
    def setUp(self):

        config_data = {
            "i_file": "./tests/exp_data/B95_Ch0_IDRest_107.ibw",
            "v_file": "./tests/exp_data/B95_Ch3_IDRest_107.ibw",
            "i_unit": "pA",
            "v_unit": "mV",
            "t_unit": "s",
            "dt": 0.00025,
            "ljp": 14.0,
        }

        self.recording = Step(
            config_data,
            igor_reader(config_data)[0],
            protocol_name="Step",
            efel_settings={}
        )

    def test_step_ecode(self):
        self.assertTrue(isinstance(self.recording, bluepyefe.recording.Recording))
        self.assertEqual(len(self.recording.voltage), len(self.recording.current))
        self.assertEqual(len(self.recording.voltage), len(self.recording.t))
        self.assertLess(abs(self.recording.ton - 700.0), 10.0)
        self.assertLess(abs(self.recording.toff - 2700.0), 10.0)
        self.assertLess(abs(self.recording.hypamp + 0.03), 0.005)
        self.assertLess(abs(self.recording.amp - 0.033), 0.005)

    def test_get_params(self):
        params = self.recording.get_params()
        self.assertEqual(len(params), len(self.recording.export_attr))

    def test_generate(self):
        t, c = self.recording.generate()
        self.assertEqual(len(t), len(c))
        self.assertEqual(max(c), self.recording.amp + self.recording.hypamp)

    def test_in_target(self):
        self.recording.amp_rel = 100.
        self.assertTrue(self.recording.in_target(101, 2))
        self.assertFalse(self.recording.in_target(-100, 50))
        self.assertFalse(self.recording.in_target(90, 2))


class RecordingTestNWB(unittest.TestCase):

    def setUp(self):
        cell = bluepyefe.cell.Cell(name="MouseNeuron")
        file_metadata = {
                        "filepath": "./tests/exp_data/hippocampus-portal/99111002.nwb",
                        "i_unit": "A",
                        "v_unit": "V",
                        "t_unit": "s",
                        "ljp": 0.0,
                        "protocol_name": "Step",
                    }
        cell.read_recordings(protocol_data=[file_metadata], protocol_name="Step")
        self.cell = cell

    def test_set_autothreshold(self):
        """Test the auto_threshold detection in Recording."""
        assert self.cell.recordings[0].auto_threshold == approx(4.999999)
        assert self.cell.recordings[15].auto_threshold == approx(26.5)

    def test_compute_spikecount(self):
        """Test Recording.compute_spikecount()."""
        assert self.cell.recordings[1].spikecount == 2
        assert_array_almost_equal(self.cell.recordings[1].peak_time, [85.4, 346.1])


if __name__ == "__main__":
    unittest.main()



#!/bin/env python

from __future__ import print_function

import sys
import os
import contextlib
import datetime

metadata_template = \
    """---
packageurl: https://bbpteam.epfl.ch/repository/devpi/bbprelman/dev/bluepyefe
major: {major_version}
description: Get efeatures from experimental data
repository: https://bbpcode.epfl.ch/code/#/admin/projects/sim/BluePyEfe
externaldoc: https://bbpcode.epfl.ch/documentation/#BluePyEfe
updated: {date}
maintainers: Werner Van Geit
name: BluePyEfe
license: BBP-internal-confidential
version: {version}
contributors: Christian Roessert, Werner Van Geit,  BBP
minor: {minor_version}
---
"""


@contextlib.contextmanager
def cd(dir_name):
    """Change directory"""
    old_cwd = os.getcwd()
    os.chdir(dir_name)
    try:
        yield
    finally:
        os.chdir(old_cwd)


def main():
    """Main"""
    doc_dir = sys.argv[1]

    doc_dir = os.path.abspath(doc_dir)

    with cd(doc_dir):
        print('Reading BluePyEfe version ...')
        import bluepyefe
        bluepyefe_version = bluepyefe.__version__
        bluepyefe_major_version = bluepyefe_version.split('.')[0]
        bluepyefe_minor_version = bluepyefe_version.split('.')[1]
        print('BluePyEfe version is: %s' % bluepyefe_version)

        finished_filename = '.doc_version'

        if os.path.exists(finished_filename):
            os.remove(finished_filename)

        metadata_filename = 'metadata.md'

        metadata_content = metadata_template.format(
            major_version=bluepyefe_major_version,
            minor_version=bluepyefe_minor_version,
            date=datetime.datetime.now().strftime("%d/%m/%y"),
            version=bluepyefe_version)

        print('Created metadata: %s' % metadata_content)

        with open(metadata_filename, 'w') as metadata_file:
            metadata_file.write(metadata_content)

        print('Wrote metadata to: %s' % metadata_filename)

        with open(finished_filename, 'w') as finished_file:
            finished_file.write(bluepyefe_version)

        print('Wrote doc version info to: %s' % finished_filename)


if __name__ == '__main__':
    main()



"""bluepyefe.cell tests"""

import unittest

import bluepyefe.cell
import bluepyefe.recording
from bluepyefe.rheobase import compute_rheobase_absolute


class CellTest(unittest.TestCase):
    def setUp(self):

        self.cell = bluepyefe.cell.Cell(name="MouseNeuron")

        file_metadata = {
            "i_file": "./tests/exp_data/B95_Ch0_IDRest_107.ibw",
            "v_file": "./tests/exp_data/B95_Ch3_IDRest_107.ibw",
            "i_unit": "pA",
            "v_unit": "mV",
            "t_unit": "s",
            "dt": 0.00025,
            "ljp": 14.0,
        }

        self.cell.read_recordings(protocol_data=[file_metadata], protocol_name="IDRest")

        self.cell.extract_efeatures(
            protocol_name="IDRest", efeatures=["Spikecount", "AP1_amp"]
        )

    def test_efeature_extraction(self):
        recording = self.cell.recordings[0]
        self.assertEqual(2, len(recording.efeatures))
        self.assertEqual(recording.efeatures["Spikecount"], 9.0)
        self.assertLess(abs(recording.efeatures["AP1_amp"] - 66.4), 2.0)

    def test_amp_threshold(self):
        recording = self.cell.recordings[0]
        compute_rheobase_absolute(self.cell, ["IDRest"])
        self.cell.compute_relative_amp()
        self.assertEqual(recording.amp, self.cell.rheobase)
        self.assertEqual(recording.amp_rel, 100.0)


if __name__ == "__main__":
    unittest.main()



"""bluepyefe.target tests"""

import unittest
import numpy
import math

from bluepyefe.target import EFeatureTarget


class TestEFeatureTarget(unittest.TestCase):

    def setUp(self):
        self.target = EFeatureTarget(
            efeature_name='test_spikecount',
            efel_feature_name='Spikecount',
            protocol_name='IDRest',
            amplitude=150.,
            tolerance=10.
        )

    def test_init(self):
        self.assertEqual(self.target.sample_size, 0)

    def test_nan(self):
        self.target.append(numpy.nan)
        self.target.append(math.nan)
        self.assertEqual(self.target.sample_size, 0)

    def test_append_clear(self):
        self.target.append(1.)
        self.target.append(2.)
        with self.assertRaises(TypeError) as context:
            self.target.append([1.])
        self.assertTrue('Expected value' in str(context.exception))
        self.assertEqual(self.target.sample_size, 2)
        self.target.clear()
        self.assertEqual(self.target.sample_size, 0)

    def test_mean_std(self):
        self.assertTrue(numpy.isnan(self.target.mean))
        self.assertTrue(numpy.isnan(self.target.std))
        self.target.append(1.)
        self.target.append(2.)
        self.assertEqual(self.target.mean, 1.5)
        self.assertEqual(self.target.std, 0.5)

    def test_dict(self):
        self.target.append(1.)
        self.target.append(2.)
        dict_form = self.target.as_dict()
        self.assertEqual(len(dict_form), 5)
        self.assertEqual(len(dict_form['val']), 2)
        self.assertEqual(len(dict_form['efel_settings']), 0)

    def test_str(self):
        print(self.target)
        self.target.append(1.)
        self.target.append(2.)
        print(self.target)


if __name__ == "__main__":
    unittest.main()



from bluepyefe.ecode import tools
import numpy as np
from numpy.testing import assert_array_almost_equal, assert_almost_equal


def test_scipy_signal2d():
    np.random.seed(42)
    data = np.random.uniform(-1, 1, 10)
    res = tools.scipy_signal2d(data, 85)
    assert_array_almost_equal(
        res,
        [
            0.2022300234864176,
            0.1973169683940732,
            0.1973169683940732,
            0.1973169683940732,
            0.1973169683940732,
            0.1973169683940732,
            0.1973169683940732,
            0.2022300234864176,
            0.2022300234864176,
            0.2022300234864176,
        ],
    )


def test_base_current():
    np.random.seed(42)
    data = np.random.uniform(-1, 1, 10)
    base = tools.base_current(data)
    assert_almost_equal(base, 0.1973169683940732)



"""bluepyefe.protocol tests"""

import unittest
import glob

from bluepyefe.translate_legacy_config import translate_legacy_config
import bluepyefe.extract
import bluepyefe.tools


class TestTranslateLegacyConfig(unittest.TestCase):

    def setUp(self):

        files_metadata = {}
        for cell_name in ["B6", "B8"]:
            for path in glob.glob(f"./tests/exp_data/{cell_name}/{cell_name}_Ch0_IDRest_*.ibw"):

                if cell_name not in files_metadata:
                    files_metadata[cell_name] = {"experiments": {"IDRest": {"files": []}}}

                files_metadata[cell_name]["experiments"]["IDRest"]["files"].append(
                    {
                        "i_file": path,
                        "v_file": path.replace("Ch0", "Ch3"),
                        "i_unit": "pA",
                        "v_unit": "mV",
                        "t_unit": "s",
                        "dt": 0.00025,
                        "ljp": 14.0,
                    }
                )

        self.config = {
            "cells": files_metadata,
            "features": {"IDRest": ["AP_amplitude", "Spikecount"]},
            "options": {
                "target": [150, 200, 250],
                "tolerance": [20, 20, 20],
                "onoff": {"IDRest": [700, 2700]},
                "expthreshold": ["IDRest"]
            },
            "path": "./",
        }

    def test_translate(self):
        translated_config = translate_legacy_config(self.config)

        cells = bluepyefe.extract.read_recordings(
            files_metadata=translated_config["files_metadata"])

        cells = bluepyefe.extract.extract_efeatures_at_targets(
            cells=cells, targets=translated_config["targets"]
        )

        bluepyefe.extract.compute_rheobase(cells, protocols_rheobase=["IDRest"])

        self.assertEqual(len(cells), 2)
        self.assertEqual(len(cells[0].recordings), 5)
        self.assertEqual(len(cells[1].recordings), 5)

        self.assertLess(abs(cells[0].rheobase - 0.119), 0.01)
        self.assertLess(abs(cells[1].rheobase - 0.0923), 0.01)


if __name__ == "__main__":
    unittest.main()



"""bluepyefe.ecode.sAHP tests"""

import unittest
import glob
import json

import bluepyefe.extract
import bluepyefe.tools
from tests.utils import download_sahp_datafiles


def get_sahp_config(absolute_amplitude=False):
    download_sahp_datafiles()

    interesting_efeatures = {
        "Spikecount": {},
        "mean_frequency": {},
        "ISI_CV": {},
        "AP1_amp": {},
        "AP_width": {},
    }

    files_metadata1 = []
    for file in glob.glob("./tests/exp_data/X/X_sAHP_ch0_*.ibw"):
        files_metadata1.append(
            {
                "i_file": file,
                "v_file": file.replace("ch0", "ch1"),
                "i_unit": "A",
                "v_unit": "V",
                "t_unit": "ms",
                "dt": 0.25,
                "ljp": 14,
                "ton": 10, # in ms
                "tmid": 260, # in ms
                "tmid2": 360, # in ms
                "toff": 1360, # in ms
            }
        )
    files_metadata2 = []
    for file in glob.glob("./tests/exp_data/X/X_IDthresh_ch0_*.ibw"):
        files_metadata2.append(
            {
                "i_file": file,
                "v_file": file.replace("ch0", "ch1"),
                "i_unit": "A",
                "v_unit": "V",
                "t_unit": "ms",
                "dt": 0.25,
                "ljp": 14,
            }
        )

    files_metadata = {
        "MouseNeuron1": {"sAHP": files_metadata1, "IDthresh": files_metadata2},
    }

    if absolute_amplitude:
        targets = {
            "sAHP": {
                "amplitudes": [0.315, 0.225, 0.5, 0.69, 0.41, 0.595],
                "tolerances": [0.1],
                "efeatures": interesting_efeatures,
                "location": "soma",
            }
        }

    else:
        targets = {
            "sAHP": {
                "amplitudes": [285, 200, 450, 625, 370, 540],
                "tolerances": [10.0],
                "efeatures": interesting_efeatures,
                "location": "soma",
            }
        }

    return files_metadata, bluepyefe.extract.convert_legacy_targets(targets)


class SAHPTest(unittest.TestCase):
    def test_extract_sahp(self):
        for absolute_amplitude in [True, False]:
            with self.subTest(absolute_amplitude=absolute_amplitude):
                self.run_test_with_absolute_amplitude(absolute_amplitude)

    def run_test_with_absolute_amplitude(self, absolute_amplitude):
        files_metadata, targets = get_sahp_config(absolute_amplitude)

        cells = bluepyefe.extract.read_recordings(files_metadata=files_metadata)

        cells = bluepyefe.extract.extract_efeatures_at_targets(
            cells=cells, targets=targets
        )

        bluepyefe.extract.compute_rheobase(cells, protocols_rheobase=["IDthresh"])

        self.assertEqual(len(cells), 1)
        self.assertEqual(len(cells[0].recordings), 24)
        self.assertLess(abs(cells[0].rheobase - 0.1103), 0.01)

        # amplitude test for one recording
        # sort the recordings because they can be in any order,
        # and we want to select the same one each time we test
        sahp_recs = [rec for rec in cells[0].recordings if rec.protocol_name == "sAHP"]
        rec1 = sorted(sahp_recs, key=lambda x: x.amp2)[1]
        self.assertLess(abs(rec1.amp - 0.0953), 0.01)
        self.assertLess(abs(rec1.amp2 - 0.3153), 0.01)
        self.assertLess(abs(rec1.amp_rel - 86.4), 0.1)
        self.assertLess(abs(rec1.amp2_rel - 285.8), 0.1)


        protocols = bluepyefe.extract.group_efeatures(
            cells,
            targets,
            use_global_rheobase=True,
            protocol_mode="mean",
            absolute_amplitude=absolute_amplitude
        )

        _ = bluepyefe.extract.create_feature_protocol_files(
            cells=cells, protocols=protocols, output_directory="MouseCells_sAHP"
        )

        for protocol in protocols:
            if protocol.name == "sAHP" and protocol.amplitude == 625:
                for target in protocol.feature_targets:
                    if target.efel_feature_name == "Spikecount":
                        self.assertEqual(target.mean, 6)
                        break

        bluepyefe.extract.plot_all_recordings_efeatures(
            cells, protocols, output_dir="MouseCells_sAHP/"
        )

        with open("MouseCells_sAHP/features.json") as fp:
            features = json.load(fp)
        with open("MouseCells_sAHP/protocols.json") as fp:
            protocols = json.load(fp)

        self.assertEqual(len(features), len(protocols))

if __name__ == "__main__":
    unittest.main()


"""bluepyefe.protocol tests"""

import unittest

from bluepyefe.ecode.step import Step
from bluepyefe.protocol import Protocol
from bluepyefe.target import EFeatureTarget


class TestProtocol(unittest.TestCase):

    def setUp(self):
        target = EFeatureTarget(
            efeature_name='test_spikecount',
            efel_feature_name='Spikecount',
            protocol_name='IDRest',
            amplitude=150.,
            tolerance=10.
        )

        self.protocol = Protocol(
            name='IDRest',
            feature_targets=[target],
            amplitude=150.,
            tolerance=10.,
            mode="mean"
        )

    def test_append_clear(self):
        rec = Step(config_data={}, reader_data={})
        rec.efeatures = {"test_spikecount": 10.}

        self.protocol.append(rec)
        self.protocol.append(rec)
        self.assertEqual(self.protocol.n_match, 2)

    def test_str(self):
        print(self.protocol)


if __name__ == "__main__":
    unittest.main()
